﻿using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;


namespace SinkingTowerDirectX
{

    class Block : Sprite
    {
        public float originalWeight;
        public float weight;
        const int deadLine = 360;
        const int fullDeadLine = 380;
        public string type;
        public int range;
        private int speed;

        public Block(Texture2D texture, Vector2 location, string name, float weight)
        {
            this.texture = texture;
            this.location = location;
            this.originalWeight = weight;
            this.weight = this.originalWeight;
            this.dead = false;
            this.type = name;
            this.height = 40;
            this.width = 40;
            this.hitbox = new Rectangle(Convert.ToInt32(location.X), Convert.ToInt32(location.Y), this.width, this.height);

            switch (name)
            {
                case "gun":
                    this.range = 200;
                    this.speed = 2;
                    break;
                case "range":
                    this.range = 600;
                    this.speed = 4;
                    break;
                default:
                    this.range = 0;
                    this.speed = 2;
                    break;
            }
        }

        new public void Draw(SpriteBatch spriteBatch)
        {
                spriteBatch.Draw(texture, location, Color.White * deadness()); 
        }

        private float deadness()
        {

            if (this.location.Y<deadLine)
            {
                return 1f;
            }
            else if (this.location.Y>fullDeadLine)
            {
                return 0f;
            }
            else 
            {
                float distance = fullDeadLine - deadLine;
                float fallen = this.location.Y - deadLine;
                return 1f - (fallen / distance);
            }
        }

        public void Update(GameTime gameTime, float sinkSpeed)
        {
            this.location.Y += sinkSpeed;

            if (location.Y > 380)
            {
                this.weight = this.originalWeight * deadness();

                if (!this.dead && this.location.Y >=fullDeadLine)
                {
                    this.dead = true;
                }
            }
            this.hitbox.X = Convert.ToInt32(this.location.X);
            this.hitbox.Y = Convert.ToInt32(this.location.Y);

            //Timespan x = gametime.ElapsedGameTime;
        }

        internal List<Bullet> Shoot(List<Enemy> enemies)
        {
            List<Bullet> bullets = new List<Bullet>();
            if (this.type == "gun" || this.type == "range")
            {
                if (enemies.Count > 0)
                {
                    var minDistance = enemies.Min(enemy => Vector2.Distance(enemy.location, this.location));
                    if (minDistance < this.range)
                    {
                        var closest = enemies.First(enemy => Vector2.Distance(enemy.location, this.location) == minDistance);
                        Vector2 direction = Vector2.Normalize(closest.location - (this.location+ new Vector2(20,20)));
                        bullets.Add(new Bullet(location + new Vector2(20, 20), direction * speed));
                    }
                }
            }
            else if (this.type =="multi")
            {
                bullets.Add( new Bullet(location + new Vector2(20, 20), Vector2.Normalize(new Vector2(-1,0))*2));
                bullets.Add( new Bullet(location + new Vector2(20, 20), Vector2.Normalize(new Vector2(-1,-1))*2));
                bullets.Add( new Bullet(location + new Vector2(20, 20), Vector2.Normalize(new Vector2(0,-1))*2));
                bullets.Add( new Bullet(location + new Vector2(20, 20), Vector2.Normalize(new Vector2(1,-1))*2));
                bullets.Add( new Bullet(location + new Vector2(20, 20), Vector2.Normalize(new Vector2(1,0))*2));
                bullets.Add( new Bullet(location + new Vector2(20, 20), Vector2.Normalize(new Vector2(1,1))*2));
                bullets.Add( new Bullet(location + new Vector2(20, 20), Vector2.Normalize(new Vector2(-1,1))*2));
                

            }
            return bullets;
            
        }
    }
}
