﻿#region Using Statements
using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Media;
#endregion

namespace SinkingTowerDirectX
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Game
    {
        const int groundLine = 380;
        
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        Texture2D block;
        Texture2D background;
        SpriteFont mainFont;

        SoundEffect build;
        SoundEffect fail;
        Song music;

        bool wasClicked;
        Tower[] towers;
        List<Enemy> enemies = new List<Enemy>();
        List<Spawner> spawns = new List<Spawner>();

        int money;
        float score;
        bool gameOver;
        List<Button> buttons;

        private bool songstart;
        private Texture2D moneyTexture;
        private Texture2D balloon;
        private Texture2D tank;
        private SoundEffect hit;
        private List<Bullet> bullets;
        private Texture2D bulletTexture;
        private Texture2D longTexture;
        private Texture2D multiTexture;
        private float eventSeconds;
        private bool eventFlipped;

        private SoundEffect bulletHit;

        Button selectedButton;
        private Texture2D selectedTexture;
        private int selectedCost;
        private float selectedWeight;
        private string selectedName;
        private Texture2D baseTexture;
        private SoundEffect selectSound;
        private Texture2D ufoTexture;
        
          
        public Game1()
            : base()
        {


            graphics = new GraphicsDeviceManager(this);
            
            graphics.PreferredBackBufferHeight = 600;
            graphics.PreferredBackBufferWidth = 800;


            graphics.ApplyChanges();

            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {


            IsMouseVisible = true;
            bullets = new List<Bullet>();
            eventSeconds = 0;
            gameOver = false;

            money = 50;
            score = 0;

            selectedButton = null;
            this.Window.Title = "Sinking tower";

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);
            block = Content.Load<Texture2D>("tile");
            background = Content.Load<Texture2D>("background");
            moneyTexture = Content.Load<Texture2D>("money");
            balloon = Content.Load<Texture2D>("balloon");
            tank = Content.Load<Texture2D>("tank");
            bulletTexture = Content.Load<Texture2D>("bullet");
            baseTexture = Content.Load<Texture2D>("base");
            multiTexture = Content.Load<Texture2D>("multi");
            longTexture = Content.Load<Texture2D>("longshot");
            ufoTexture = Content.Load<Texture2D>("ufo");

            
            //build = Content.Load<SoundEffect>("build.wav");
            //fail = Content.Load<SoundEffect>("fail.wav");
            //hit = Content.Load<SoundEffect>("hit.wav");
            //bulletHit = Content.Load<SoundEffect>("bulletHit.wav");
            //selectSound = Content.Load<SoundEffect>("selectTower.wav");
            //music = Content.Load<Song>("backgroundMusic.wav");

            mainFont = Content.Load<SpriteFont>("SpriteFont1");

            selectedTexture = block;
            selectedCost = 10;
            selectedWeight = 1;
            selectedName = "gun";

            buttons = new List<Button>();
            buttons.Add(new Button(baseTexture, new Vector2(20, 500), "base", 5, 1));
            buttons.Add(new Button(block, new Vector2(110, 500),"gun",5,3));
            buttons.Add(new Button(multiTexture, new Vector2(200, 500), "multi", 25, 2));
            buttons.Add(new Button(longTexture, new Vector2(290, 500), "range", 30, 2));
            buttons.Add(new Button(moneyTexture, new Vector2(380, 500), "cash", 30, 4));
            buttons.Add(new Button(balloon, new Vector2(470, 500), "lift", 55, -5));
            buttons.Add(new Button(moneyTexture, new Vector2(560, 500), "cash", 75, 1));

            //buttons[0].isHighlighted = true;

            wasClicked = false;
            towers = new Tower[10];

            for (int i = 0; i < towers.Length; i++)
            {
                towers[i] = new Tower();
                towers[i].blocks.Add(new Block(baseTexture, new Vector2(i * 40 + 200, 340),"base",-0.1f));
            }
            towers[0].blocks.Add(new Block(block, new Vector2(200, 300), "gun", 1f));
            towers[9].blocks.Add(new Block(balloon, new Vector2( 560, 300),"balloon",-5f));
            towers[9].blocks.Add(new Block(moneyTexture, new Vector2(560, 260),"cash",2f));

            spawns = new List<Spawner>();

            spawns.Add(new Spawner(4f, 0, "tank", tank));
            spawns.Add(new Spawner(21f,0, "ufo", ufoTexture));
            spawns.Add(new Spawner(3.5f, 45, "tank", tank));
            spawns.Add(new Spawner(3.25f, 60, "tank", tank));
            spawns.Add(new Spawner(7f, 60, "ufo", ufoTexture));
            spawns.Add(new Spawner(1f, 90, "tank", tank));
            spawns.Add(new Spawner(1f, 120, "tank", tank));
            spawns.Add(new Spawner(2f, 150, "ufo", ufoTexture));
            spawns.Add(new Spawner(1f, 180, "tank", tank));
            spawns.Add(new Spawner(2.1f, 210, "ufo", ufoTexture));
            spawns.Add(new Spawner(2.2f, 240, "ufo", ufoTexture));
            spawns.Add(new Spawner(0.75f, 300, "tank", tank));
            spawns.Add(new Spawner(0.7f, 360, "tank", tank));
            spawns.Add(new Spawner(1f, 360, "ufo", ufoTexture));
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            spriteBatch.Dispose();
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            /* CONFIG */
            if (!songstart)
            {
                MediaPlayer.IsRepeating = true;
                MediaPlayer.IsShuffled = true;
                
                //MediaPlayer.Play(music);
                MediaPlayer.IsRepeating = true;

                songstart = true;
            }


            eventFlipped = false;
            eventSeconds += gameTime.ElapsedGameTime.Milliseconds / 1000f;
            if (eventSeconds > 1)
            {
                eventSeconds -= 1;
                eventFlipped = true;
                
            }

            /* PLAYER INPUT */
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed || Keyboard.GetState().IsKeyDown(Keys.Escape))
                Exit();

            if (Mouse.GetState().LeftButton == ButtonState.Pressed)
            {
                if (wasClicked == false && !gameOver)
                {
                    int towerID = getTowerId(towers.Length, Mouse.GetState().Position.X);

                    //Build a tower
                    if (towerID != -1 && Mouse.GetState().Position.Y <= groundLine &&  !towers[towerID].dead && money >= selectedCost)
                    {
                        towers[towerID].blocks.Add(new Block(selectedTexture, getTowerVector(towers, Mouse.GetState().Position.X), selectedName, selectedWeight));
                        //build.Play();
                        money -= selectedCost;
                    }
                    else if (selectedButton != null && selectedButton.cost <= money)
                    {
                        //selectSound.Play();
                        selectedTexture = selectedButton.texture;
                        selectedName = selectedButton.name;
                        selectedCost = selectedButton.cost;
                        selectedWeight = selectedButton.weight;
                    }
                    else
                    {
                        //fail.Play();
                    }
        
                }
                wasClicked = true;
            }
            else
            { 
                wasClicked = false; 
            }

            if (!gameOver)
            {
                /* Update Loops */

                foreach (Tower t in towers)
                {
                    t.Update(gameTime);
                    if (eventFlipped)
                    {
                        t.Shoot(enemies, bullets, bulletTexture);
                        money += t.Income();
                    }
                }

                foreach (Enemy e in enemies)
                {
                    if (!e.dead)
                    {
                        e.Update(gameTime);

                        //Check if it hits a tower
                        int towerId = getTowerId(towers.Length, Convert.ToInt32(e.location.X + 20));
                        if (towerId != -1 && !towers[towerId].dead && towers[towerId].intersects(e))
                        {
                            e.dead = true;
                            towers[towerId].TakeDamage(e.damage);
                            //hit.Play();
                        }
                    }
                }

                foreach (Bullet b in bullets)
                {
                    b.Update(gameTime);
                    int hit = b.Hit(enemies);
                    
                    if (hit>0)
                    {
                        score += hit;
                        //bulletHit.Play();
                    }
                }

                foreach (Spawner s in spawns)
                {
                    var newEnemies = s.Update(gameTime);
                    foreach (Enemy e in newEnemies)
                    {
                        enemies.Add(e);
                    }
                }

                /* CLEAN LISTS */
                enemies = enemies.Where(enemy => !enemy.dead).ToList();
                bullets = bullets.Where(bullet => !bullet.dead 
                                                  && bullet.location.X > -bullet.width
                                                  && bullet.location.X < 800 + bullet.width
                                                  && bullet.location.Y > bullet.height
                                                  && bullet.location.Y < groundLine).ToList();
            }


            /* check if the game is over! */
            bool endGame = true;
            foreach (Tower t in towers)
            {
                if (!t.dead)
                {
                    endGame = false;
                }
            }
            if (endGame)
            {
                gameOver = true;
            }


            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);
            

            spriteBatch.Begin(SpriteSortMode.Immediate, BlendState.AlphaBlend);
            spriteBatch.Draw(background, Vector2.Zero, Color.White);

            selectedButton = null;
            Rectangle testMouse = new Rectangle(Convert.ToInt32(Mouse.GetState().Position.X),Convert.ToInt32(Mouse.GetState().Position.Y),1,1);
            foreach (Button button in buttons)
            {
                
                if (button.hitBox.Intersects(testMouse))
                {
                    selectedButton = button;
                    button.isHighlighted = true;
                }
                else
                {
                    button.isHighlighted = false;
                }
                
                button.Draw(spriteBatch, mainFont, money);
            }
            
            
            foreach (Tower t in towers)
            { 
                t.Draw(spriteBatch); 
            }



            if (Mouse.GetState().Position.Y<=groundLine)
            { 
                spriteBatch.Draw(selectedTexture, getTowerVector(towers,Mouse.GetState().Position.X), getTowerColor(towers,Mouse.GetState().Position.X));
            }


            spriteBatch.DrawString(mainFont, String.Format("Money: {0}",money),new Vector2(650,500),Color.Black);
            spriteBatch.DrawString(mainFont, String.Format("Score: {0}", Convert.ToInt32(score)), new Vector2(650, 530), Color.Black);

            if(gameOver)
            {
                spriteBatch.DrawString(mainFont, String.Format("Game Over!", money), new Vector2(300, 100), Color.Black);
            }
            
            spriteBatch.End();
            spriteBatch.Begin(SpriteSortMode.Immediate, BlendState.NonPremultiplied);
            foreach (Enemy e in enemies)
            {
                e.Draw(spriteBatch);
            }
            foreach (Bullet b in bullets)
            {
                b.Draw(spriteBatch);
            }
            spriteBatch.End();
            base.Draw(gameTime);
        }




        private Vector2 getTowerVector(Tower[] towers, int x)
        {
            int towerId = getTowerId(towers.Length, x);
            float y = 0;

            if (towerId>=0 && towerId<towers.Length && !towers[towerId].dead)
            {
                y = towers[towerId].getMaxY();   
            }

            if (towerId == -1 || towers[towerId].dead)
            {
                return new Vector2(Convert.ToInt32((x /40)*40), 340);
            }
            else  
            { 
                return new Vector2(towerId*40+200,y-40);
            }
        }

        private Color getTowerColor(Tower[] towers, int x)
        {
            int towerId = getTowerId(towers.Length, x);

            if (towerId != -1 && !towers[towerId].dead && money >= 10)
            {
                return Color.SpringGreen * 0.6f;
            }
            else
            {
                return Color.Pink * 0.6f;
            }
        }

        private int getTowerId(int towerLength, int x)
        {
            int towerId = Convert.ToInt32((x - 200) / 40);

            if (towerId >= 0 && towerId < towerLength)
            {
                return towerId;
            }

            else
            {
                return -1;
            }
        }
    }
}
