﻿using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SinkingTowerDirectX
{
    class Tower
    {
        public List<Block> blocks;
        float damage;

        public Tower()
        {
            blocks = new List<Block>();
            damage = 0f;
        }

        public void Draw(SpriteBatch spriteBatch)
        {
            foreach(Block b in blocks)
            {
                b.Draw(spriteBatch);
            }
        }

        public void Update(GameTime gameTime)
        {

            float totalWeight = 0.0f;

            float damageWeight = 0f;
            if (damage > 0)
            {
                damageWeight = MathHelper.Max(3,damage/200) * (float)gameTime.ElapsedGameTime.TotalSeconds * 60;
                damage -= MathHelper.Min(damage, damageWeight);
            }
            
            foreach (Block b in blocks)
            {
                totalWeight += b.weight;
            }

            totalWeight *= (float)gameTime.ElapsedGameTime.TotalSeconds * 60;
            totalWeight += damageWeight;

            foreach (Block b in blocks)
            {
                b.Update(gameTime, 0.005f*MathHelper.Max(totalWeight+1,1));

            }

            blocks = blocks.Where(block => !block.dead).ToList();

            
        }


        internal float getMaxY()
        {
            float y = 800;
            foreach (Block b in blocks)
            {
                if (b.location.Y < y)
                {
                    y = b.location.Y;
                }
            }
            return y;
        }

        public bool dead 
        {
            get
            {
                foreach (Block b in blocks)
                {
                    if (!b.dead)
                    {
                        return false;
                    }
                    
                }

                return true;
            }        
        }

        public void TakeDamage(float damage)
        {
            this.damage+=damage;
        }

        public void Shoot(List<Enemy> enemies, List<Bullet> bullets, Texture2D bulletTexture)
        {
            foreach (Block b in blocks)
            {
                if (b.type == "gun" || b.type=="range" || b.type=="multi")
                {
                    var bullet = b.Shoot(enemies);
                    
                    foreach (var newBullet in bullet)
                    {
                        newBullet.texture = bulletTexture;
                        bullets.Add(newBullet);
                    }
                }
            }
        }

        internal int Income()
        {
            int income = 0;
            foreach (Block block in blocks)
            {
                if (block.type == "cash")
                {
                    income += 1;
                }
                
            }
            return income;

        }

        internal bool intersects(Enemy e)
        {
            foreach (Block b in blocks)
            {
                if (e.hitbox.Intersects(b.hitbox))
                {
                    return true;
                }
             

            }
            return false;
        }
    }
}
